from .models import DjangoRunQueue
def add(taskname,queuename,taskArgs):
    try:
        rec=DjangoRunQueue.objects.create(TaskName=taskname,TaskQueue=queuename,TaskArgs=taskArgs)
        return 'Task %d has been added' % rec.id
    except:
        return 'Failed to add task'


def add_unique(taskname,queuename,taskArgs):
    try:
        if not DjangoRunQueue.objects.filter(TaskName=taskname, TaskQueue=queuename,
                                             TaskArgs=taskArgs, Proccessed=False).exists():
            rec = DjangoRunQueue.objects.create(TaskName=taskname, TaskQueue=queuename, TaskArgs=taskArgs)
            return 'Task %d has been added' % rec.id

    except:
        return 'Failed to add task'

def get_result(taskid):
    try:
        rec=DjangoRunQueue.objects.select_for_update(nowait=True).filter(id=taskid)
        if rec.Proccessed:
            return rec.TaskResults
        else:
            return "Not proccessed Yet"
    except:
        return "Still being proccessed"


# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-25 11:47
from __future__ import unicode_literals

from django.db import migrations, models
import picklefield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DjangoRunQueue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('TaskName', models.CharField(max_length=100)),
                ('TaskQueue', models.CharField(max_length=100)),
                ('TaskArgs', picklefield.fields.PickledObjectField(editable=False)),
                ('TaskResults', picklefield.fields.PickledObjectField(editable=False)),
                ('Proccessed', models.BooleanField(default=False)),
            ],
        ),
    ]



from django.core.management.base import BaseCommand
from django_run_queues.models import *
from django.conf import settings
import importlib
from django.db import transaction

tasklib = importlib.import_module(settings.DRQ_TASKS_LIB)


class Command(BaseCommand):
    help = 'Runs Tasks queued on a specific Queue Name for Django Run-Queue'

    def add_arguments(self, parser):
        parser.add_argument('--queue', dest='queue_name', required=True)
    @transaction.atomic()
    def handle(self, *args, **options):
        tasklist = DjangoRunQueue.objects.select_for_update().\
                       filter(TaskQueue=options['queue_name'], Proccessed=False)
        for task in tasklist:
            print('running task %d' % task.id)
            try:
                func = getattr(tasklib, task.TaskName)
                task.TaskResults = func(**task.TaskArgs)
                task.Proccessed = True
                task.save()
            except Exception as e:
                task.TaskResults = "Taskid %d failed with %s" % (task.id, e)
                task.Proccessed = True
                task.save()
                print(e)
            print(task.TaskResults)



from django.core.management.base import BaseCommand
from django_run_queues.models import *
from django.conf import settings
from crontab import CronTab


class Command(BaseCommand):
    help = 'Creates Crontab entries to schedule queue runs'

    def handle(self, *args, **options):

        tab = CronTab(user=True)
        del_list=[]
        for item in tab:
            if 'RunQueue' in item.command:
                del_list.append(item)
        for item in del_list:
            tab.remove(item)
        queue_details = settings.RUN_QUEUE_SCHEDULE

        if 'environment' not in queue_details:
            environ = ''
        else:
            environ = queue_details['environment']
        if 'settings' in queue_details:
            dj_settings ='--settings=%s' % queue_details['settings']
        else:
            dj_settings = ''

        for queue in queue_details['queues']:
            for times in queue_details['queues'][queue]:
                job = tab.new(command='. %s/bin/activate python %s/manage.py RunQueue --queue %s %s' % (environ,queue_details['base_dir'],queue,dj_settings))
                job.setall(times)
                print(job)
                if not job.is_valid():
                    tab.remove(job)
                    print('Command rejected %s %s' % (queue, times))
        tab.write(user=True)



import os
from setuptools import find_packages, setup

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-run-queues',
    version='0.1.4',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['django-picklefield','python-crontab'],
    license='BSD License',  # example license
    description='A simple Django sequencial task queue system',
    url='https://willowtreesoftware.com/',
    author='Marc Nealer',
    author_email='marc@willowtreesoftware.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.10',  # replace "X.Y" as appropriate
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        # Replace these appropriately if you are stuck on Python 2.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)


from __future__ import unicode_literals
from picklefield.fields import PickledObjectField
from django.db import models

# Create your models here.
class DjangoRunQueue(models.Model):
    TaskName=models.CharField(max_length=100)
    TaskQueue= models.CharField(max_length=100)
    TaskArgs=PickledObjectField()
    TaskResults=PickledObjectField()
    Proccessed=models.BooleanField(default=False)
    def __str__(self):
        return self.TaskName